import sys
import os
import logging
import itertools
import re
from inspect import getdoc
from contextlib import suppress
from logging.config import fileConfig
from termcolor import colored
from functools import partial

from cli.util import get_version_info
from cli.apiexternal import configure
from cli.docopt_command import DocoptDispatcher, get_docopt_help
from cli.image import Image
from cli.reputation import Reputation
from cli.integration import DockerRegistryIntegration, CloudIntegration
from cli.workspace import Workspace
from cli.agent import Agent
from cli.collector import Collector
from cli.file_event_search import (
    CloudLogs,
    SysLog,
    LogFiles,
    VersionedFiles,
    WindowsEvents,
)
from cli.inspections import file_inspection


log = logging.getLogger()

############################# configure logger  ################################
fileConfig(os.path.join(os.path.dirname(os.path.abspath(__file__)), "logger.ini"))

logging.addLevelName(
    logging.INFO,
    colored(f"==>", "cyan", attrs=["bold"]),
)
logging.addLevelName(
    logging.WARNING,
    colored(
        f"[{logging.getLevelName(logging.WARNING).ljust(9)}]", "yellow", attrs=["bold"]
    ),
)
logging.addLevelName(
    logging.ERROR,
    colored(f"[{logging.getLevelName(logging.ERROR).ljust(9)}]", "red", attrs=["bold"]),
)

################################################################################


def main():
    dispatcher = DocoptDispatcher(
        TopLevelCommand, version=get_version_info(), options_first=True
    )
    handler, options = dispatcher.parse(sys.argv[1:])
    handler(TopLevelCommand(), options)


class TopLevelCommand:
    """
    Usage:
        vp-cli [COMMAND] [ARGS...] [OPTIONS]
        vp-cli -h |--help
        vp-cli --version

    Commands:
        image       Image scan services
        registry    Docker registry integrations
        feeds       Reputation feeds
        integration Cloud integrations
        search      File and event search
        deploy      Deploy agent, collector or falco
        configure   Configure the cli
        inspection  Infrastructure-as-Code inspection
    """

    def configure(self, options):
        """
        Configure the cli to interact with a the VantagePoint API

        Usage:
            configure

        """
        return configure()

    def image(self, options):
        """
        Image scan services

        Usage:
            image add <registry/repo:tag> [-s=SUBSCRIBE] [-d=<Dockerfile>] [-a=TEXT]
            image (list | ls)
            image analyze <registry/repo:tag> [-d=<Dockerfile>] [-t=TIMEOUT] [-o=OUTPUT_JSON] [-e=EXIT_CODE] [-f]
            image (delete | del) IMAGE_DIGEST [-f=FORCE]

        Options:
            -f, --force                         Force reanalysis of image
            -s, --autosubscribe                 Analyse updates to image tag
            -d, --dockerfile <Dockerfile>       Submit image's docker file for analysis
            -a, --annotations TEXT
            -t, --timeout TIMEOUT               Wait timeout
            -o, --output OUTPUT_JSON            Path to save scan output
            -e, --exit-code EXIT_CODE           Exit code
        """
        if options["add"]:
            command = partial(
                Image.add,
                options["<registry/repo:tag>"],
                options["--dockerfile"],
                options["--force"],
                options["--annotations"],
                options["--autosubscribe"],
            )

        if any([options["list"], options["ls"]]):
            command = Image.list_

        if options["analyze"]:
            with suppress(TypeError):
                options["--timeout"] = float(options["--timeout"])

            command = partial(
                Image.analyze,
                options["<registry/repo:tag>"],
                options["--timeout"] or 3600,
                options["--dockerfile"],
                options["--output"],
                options["--exit-code"],
                options["--force"],
            )

        if any([options["delete"], options["del"]]):
            command = partial(Image.delete, options["IMAGE_DIGEST"])

        try:
            print(command())
        except Exception as e:
            log.error(str(e))

    def registry(self, options):
        """
        Docker registry integrations

        Usage:
            registry add NAME REGISTRY_ADDRESS REPOSITORY [-u=USERNAME] [-p=PASSWORD] [-t=TYPE] [-x=LOOKUP_TAG]
            registry (list | ls)
            registry (get | describe) ID
            registry edit ID
            registry (delete | del) ID

        Options:
            -u, --username USERNAME              Registry username
            -p, --password PASSWORD              Registry password
            -t, --type TYPE                      Docker registry type (docker_hub|aws|gcr|azure|docker_v2) [default: docker_v2]
            -x, --lookup-tag LOOKUP_TAG          Lookup tag [default: latest]
        """
        if options["add"]:
            command = partial(
                DockerRegistryIntegration.add,
                options["NAME"],
                options["REGISTRY_ADDRESS"],
                options["REPOSITORY"],
                username=options["--username"],
                password=options["--password"],
                type_=options["--type"],
                lookup_tag=options["--lookup-tag"],
            )

        if any([options["list"], options["ls"]]):
            command = partial(DockerRegistryIntegration.list_)

        if any([options["get"], options["describe"]]):
            command = partial(DockerRegistryIntegration.get, id_=options["ID"])

        if options["edit"]:
            command = partial(DockerRegistryIntegration.edit, id_=options["ID"])

        if any([options["delete"], options["del"]]):
            command = partial(DockerRegistryIntegration.delete, id_=options["ID"])

        try:
            print(command())
        except Exception as e:
            log.error(str(e))

    def feeds(self, options):
        """
        Reputation feeds

        Usage:
            feeds add -f JSON_FILE
            feeds (list | ls)
            feeds (get | describe) ID
            feeds edit ID
            feeds (delete | del) ID

        Options:
            -f, --file JSON_FILE        Path to reputation feeds list JSON file
        """

        if options["add"]:
            command = partial(Reputation.add, json_file=options["--file"])

        if any([options["list"], options["ls"]]):
            command = Reputation.list_

        if any([options["get"], options["describe"]]):
            command = partial(Reputation.get, id_=options["ID"])

        if options["edit"]:
            command = partial(Reputation.edit, id_=options["ID"])

        if any([options["delete"], options["del"]]):
            command = partial(Reputation.delete, id_=options["ID"])

        try:
            print(command())
        except Exception as e:
            log.error(str(e))

    def integration(self, options):
        """
        Integrations

        Usage:
            integration add NAME -t INTEGRATION_TYPES -p PROVIDER [--aws-role-arn=TEXT] [--flow-log-resource-type=TYPE] [--flow-log-resource-id=TEXT] [--azure-principal=TEXT] [--azure-password=PASSWORD] [--azure-tenant=TEXT]
            integration (list | ls)
            integration (get | describe) ID
            integration (delete | del) ID

        Options:
            -t, --types INTEGRATION_TYPES   Comma separated (security | cloud_log | network | flow_log)
            -p, --provider PROVIDER         Cloud provider (aws | azure)
            --aws-role-arn TEXT             AWS role arn
            --flow-log-resource-type TYPE   AWS flow log resource type (VPC | Subnet | NetworkInterface)
            --flow-log-resource-id TEXT     AWS flow log resource id
            --azure-principal TEXT          Azure principal
            --azure-password PASSWORD       Azure password
            --azure-tenant TEXT             Azure tenant
        """

        if options["add"]:
            command = partial(
                CloudIntegration.add,
                name=options["NAME"],
                provider=options["--provider"],
                types=options["--types"].split(","),
                aws_role_arn=options["--aws-role-arn"],
                flow_log_resource_type=options["--flow-log-resource-type"],
                flow_log_resource_id=options["--flow-log-resource-id"],
                azure_principal=options["--azure-principal"],
                azure_password=options["--azure-password"],
                azure_tenant=options["--azure-tenant"],
            )

        if any([options["list"], options["ls"]]):
            command = CloudIntegration.list_

        if any([options["get"], options["describe"]]):
            command = partial(CloudIntegration.get, id_=options["ID"])

        if any([options["delete"], options["del"]]):
            command = partial(CloudIntegration.delete, id_=options["ID"])

        try:
            print(command())
        except Exception as e:
            log.error(str(e))

    def config(self, options):
        """
        Modify vp-cli configurations

        Usage:
            config SUBCOMMAND [OPTIONS]

        Subcommands:
            get-workspaces  Lists available workspaces
            set-workspace   Sets user workspace
        """

        if options["SUBCOMMAND"] == "get-workspaces":
            command = Workspace.list_

        elif options["SUBCOMMAND"] == "set-workspace":
            command = partial(Workspace.set_, name=options["OPTIONS"])

        else:
            log.error(
                f"SUBCOMMAND \"{options['SUBCOMMAND']}\" not found. See vp-cli config --help."
            )
            sys.exit(1)

        try:
            print(command())
        except Exception as e:
            log.error(str(e))

    def search(self, options):
        """
        File and event search

        Usage:
            search <resource> [-q=QUERY]

        Subcommands:
            cloud_logs
            syslog
            logs
            windows_events
            versioned

        Options:
            -q, --query QUERY Search query
        """

        command_help = getdoc(self.search)
        resource = options["<resource>"]
        if resource is None:
            raise SystemExit(command_help)

        if not hasattr(Search, resource):
            print(f'Resource "{resource}" not found \n')
            raise SystemExit(command_help)

        handler = getattr(Search, resource)
        docstring = getdoc(handler)

        get_docopt_help(docstring, resource, options_first=False)

        try:
            print(handler(Search(), options["--query"]))
        except Exception as e:
            log.error(str(e))

    def deploy(self, options):
        """
        Deploy and configure agent, collector and falco

        Usage:
            deploy <resource> (linux | windows) [--computer-list=<user@hostname:port>] [--computer-list-file=<FILE>] [--use-ad] [--ad-ou-dn AD_AD] [--ad-server AD_SERVER] [--use-ssl]

        Resources:
            agent       Client agent
            collector   Client agent collector
            falco       Falco

        Options:
            --computer-list <user@hostname:port> Comma separated list. Computers on which to install agent.
            --use-ad
            --ad-ou-dn AD_AD
            --ad-server AD_SERVER
            --use-ssl
            -f, --computer-list-file <FILE> Computer list file
        """

        hosts = (
            options["--computer-list"].split(",") if options["--computer-list"] else []
        )
        if options["--computer-list-file"]:
            with open(options["--computer-list-file"], "r") as f:
                _hosts = f.read()
            hosts += _hosts.split("\n")

        for host in hosts:
            if not re.match(
                "^(?:(?:[\w.-]+@)(?:[\w-]+)(\.[\w.-]+)?(?::[0-9]{1,5})?)$", host
            ):
                log.error(
                    f'Invalid format "{host}". Host should be of format user@domain:port'
                )
                sys.exit(1)

        if options["<resource>"] == "agent":
            command = partial(
                Agent.deploy,
                "linux" if options["linux"] else "windows",
                hosts=hosts,
                use_ad=options["--use-ad"],
                ad_ou_dn=options["--ad-ou-dn"],
                ad_server=options["--ad-server"],
                use_ssl=options["--use-ssl"],
            )

        if options["<resource>"] == "collector":
            command = partial(
                Collector.deploy,
                "linux" if options["linux"] else "windows",
                hosts=hosts,
            )

        if options["<resource>"] == "falco":
            command = partial(
                Collector.deploy_falco,
                "linux" if options["linux"] else "windows",
                hosts=hosts,
            )

        try:
            print(command())
        except Exception as e:
            log.error(str(e))

    def inspection(self, options):
        """
        Static inspection

        Usage:
            inspection POLICY PATH
        """
        status = file_inspection(options['POLICY'], options['PATH'])
        if status is None:
            print(colored("ERROR: NO POLICY OR WRONG PATH", "yellow", attrs=["bold"]))
            exit_code = 1
        elif status:
            print(colored("PASSED!", "green"))
            exit_code = 0
        else:
            print(colored("FAILED", "red"))
            exit_code = 1

        sys.exit(exit_code)


class Search:
    def cloud_logs(self, query=None):
        """
        Search cloud logs

        Usage:
            search cloud_logs [-q=QUERY]

        options:
            -q, --query QUERY   Search query
        """
        return CloudLogs().search(query)

    def syslog(self, query=None):
        """
        Search syslogs

        Usage:
            search syslog [-q=QUERY]

        options:
            -q, --query QUERY Search query
        """
        return SysLog().search(query)

    def logs(self, query=None):
        """
        Search log files

        Usage:
            search logs [-q=QUERY]

        options:
            -q, --query QUERY Search query
        """
        return LogFiles().search(query)

    def windows_events(self, query=None):
        """
        Search windows events

        Usage:
            search windows_events [-q=QUERY]

        options:
            -q, --query QUERY Search query
        """
        return WindowsEvents().search(query)

    def versioned(self, query=None):
        """
        Search integrity monitored files

        Usage:
            search versioned [-q=QUERY]

        options:
            -q, --query QUERY Search query
        """
        return VersionedFiles().search(query)



import os
import logging
import sys
import requests
import subprocess
import base64
import json
import time
import getpass
from paramiko import ssh_exception
from termcolor import colored
from paramiko import client, AutoAddPolicy
from cli.apiexternal import get_vp_install_key
from cli.util import _exec_command_remote, ssh_connect


log = logging.getLogger()


class Collector:
    AGENT_NAME = "vp-collector"
    AGENT_HOME = "/opt/vantagepoint/collector"

    def _deploy_linux(self, hosts=[]):
        log.info("Setting up environment...")
        env = os.environ.copy()
        # get install key
        key = get_vp_install_key()
        env["VP_INSTALL_KEY"] = key
        # get install script
        install_script = requests.get(
            "https://artifactory.build.vantagepoint.co/artifactory/tools/collector-install.sh"
        ).text

        if hosts:
            ssh_client = client.SSHClient()
            ssh_client.load_system_host_keys()
            ssh_client.set_missing_host_key_policy(AutoAddPolicy())
            for host in hosts:
                log.info(
                    colored(
                        f'Installing collector on "{host}"', "white", attrs=["bold"]
                    )
                )
                _host, port = host.split(":") if ":" in host else (host, 22)
                user, host_name = _host.split("@")
                ssh_connect(ssh_client, user, host_name, port)
                sftp_client = ssh_client.open_sftp()
                _install_script = sftp_client.open("/tmp/vp_collector_install.sh", "w")
                _install_script.write(install_script)
                _install_script.close()
                _exec_command_remote(
                    ssh_client,
                    "chmod +x /tmp/vp_collector_install.sh && /tmp/vp_collector_install.sh",
                    host,
                    env=env,
                )
                self._update_agent_config(
                    vp_install_key=key, host=host, ssh_client=ssh_client
                )
                ssh_client.close()
        else:
            if not sys.platform != "linux":
                raise Exception("OS not supported")

            log.info("Installing collector...")
            with open("/tmp/vp_collector_install.sh", "w") as f:
                f.write(install_script)

            process = subprocess.Popen(
                "chmod +x /tmp/vp_collector_install.sh && /tmp/vp_collector_install.sh",
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
                env=env,
            )
            process.wait()

            # update config
            self._update_agent_config(key)

        return "Success!"

    def _deploy_windows(self, hosts=[]):
        raise Exception("OS not supported")

    def _deploy_falco_linux(self, hosts=[]):
        log.info("Setting up environment...")
        env = os.environ.copy()
        # get install key
        key = get_vp_install_key()
        env["VP_INSTALL_KEY"] = key
        # get install script
        install_script = requests.get(
            "https://artifactory.build.vantagepoint.co/artifactory/falco/install.sh"
        ).text

        if hosts:
            ssh_client = client.SSHClient()
            ssh_client.load_system_host_keys()
            ssh_client.set_missing_host_key_policy(AutoAddPolicy())
            for host in hosts:
                log.info(
                    colored(f'Installing falco on "{host}"', "white", attrs=["bold"])
                )
                _host, port = host.split(":") if ":" in host else (host, 22)
                user, host_name = _host.split("@")
                ssh_connect(ssh_client, user, host_name, port)
                sftp_client = ssh_client.open_sftp()
                _install_script = sftp_client.open("/tmp/vp_falco_install.sh", "w")
                _install_script.write(install_script)
                _install_script.close()
                _exec_command_remote(
                    ssh_client,
                    "chmod +x /tmp/vp_falco_install.sh && /tmp/vp_falco_install.sh",
                    host,
                    env=env,
                )
                ssh_client.close()
        else:
            if not sys.platform != "linux":
                raise Exception("OS not supported")

            log.info("Installing falco...")
            with open("/tmp/vp_falco_install.sh", "w") as f:
                f.write(install_script)

            process = subprocess.Popen(
                "chmod +x /tmp/vp_falco_install.sh && /tmp/vp_falco_install.sh",
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
                env=env,
            )
            process.wait()

    def _deploy_falco_windows(self, hosts=[]):
        raise Exception("OS not supported.")

    def _update_agent_config(self, vp_install_key, host=None, ssh_client=None):
        settings_file = f"{Collector.AGENT_HOME}/conf/settings.json"
        customer_id = base64.b64decode(vp_install_key).decode("utf-8").split("::::")[0]
        encryption_key = (
            base64.b64decode(vp_install_key).decode("utf-8").split("::::")[1]
        )
        portal_host = base64.b64decode(vp_install_key).decode("utf-8").split("::::")[2]
        workspace_id = base64.b64decode(vp_install_key).decode("utf-8").split("::::")[3]

        log.info("Updating agent config ...")

        if host:
            sftp_client = ssh_client.open_sftp()
            _config = sftp_client.open(settings_file, "r")
            config = json.loads(_config.read())
        else:
            with open(settings_file, "r") as f:
                config = json.loads(f.read())

        for k, v in {
            "customer_id": customer_id,
            "customer_encryption_key": encryption_key,
            "portal_host": portal_host,
            "workspace_id": workspace_id,
        }.items():
            config[k] = v

        if host:
            _config = sftp_client.open("/tmp/vp_collector_settings.json", "w")
            _config.write(json.dumps(config))
            _config.close()
            _exec_command_remote(
                ssh_client,
                '/bin/bash -c "%s"'
                % (
                    """
                sudo service vp-collector stop
                sleep 5s
                sudo mv /tmp/vp_collector_settings.json %s
                set -x
                sudo service vp-collector start
                        """
                    % (settings_file)
                ),
                host,
            )
        else:
            with open("/tmp/vp_collector_settings.json", "w") as f:
                f.write(json.dumps(config))

            process = subprocess.Popen(
                [
                    "bash",
                    "-c",
                    """
                sudo service vp-collector stop
                sleep 5s
                sudo mv /tmp/vp_collector_settings.json %s
                set -x
                sudo service vp-collector start
                sleep 5s
                    """
                    % (settings_file),
                    Collector.AGENT_NAME,
                ],
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
            )

            process.wait()

            for line in iter(process.stdout.readline, b""):
                sys.stdout.write(line.decode("utf-8"))

    @staticmethod
    def deploy(os, hosts=[]):
        try:
            deploy = {
                "linux": Collector()._deploy_linux,
                "windows": Collector()._deploy_windows,
            }[os]
        except KeyError:
            raise Exception('Unknown os "{os}"')

        return deploy(hosts)

    @staticmethod
    def deploy_falco(os, hosts=[]):
        try:
            deploy = {
                "linux": Collector()._deploy_falco_linux,
                "windows": Collector()._deploy_falco_windows,
            }[os]
        except KeyError:
            raise Exception('Unknown os "{os}"')

        return deploy(hosts)



from cli.apiexternal import get_workspaces
from cli.util import Formatter, update_vp_config, get_vp_config


class Workspace:
    @staticmethod
    def list_():
        workspaces = get_workspaces()
        active_workspace = get_vp_config().get("workspace", None)
        headers = ["Active", "Workspace"]
        rows = [
            ["*" if row["name"] == active_workspace else "", row["name"]]
            for row in workspaces
        ]
        return Formatter.table(headers, rows)

    @staticmethod
    def set_(name):
        avaliable_workspaces = set([w["name"] for w in get_workspaces()])
        if name not in avaliable_workspaces:
            raise Exception(
                f'Can\'t set to use workspace "{name}". See vp-cli config get-workspaces.'
            )
        update_vp_config(workspace=name)
        return "Updated!"



import os
import json
import shutil
import itertools
import base64
import dateparser
import subprocess
import tempfile
import time
import getpass
import configparser
import stat
import logging
from pathlib import Path
from io import StringIO
from activedirectory import ActiveDirectory
from paramiko import ssh_exception
from pypsrp.client import Client as WinClient
from contextlib import contextmanager
from codecs import encode
from datetime import datetime
from cli.consts import VP_CLI_VERSION, VP_CLI_DEFAULT_EDITOR
from termcolor import colored
from texttable import Texttable

CREDENTIALS_FILE = f"{Path.home()}/.vantagepoint/config"

log = logging.getLogger()

def get_version_info():
    return f"vp_cli version {VP_CLI_VERSION}"


def get_vp_config():
    config = configparser.ConfigParser()
    config.read(CREDENTIALS_FILE)
    VP_TOKEN = config.get("default", "token")
    VP_SECRET_KEY = config.get("default", "secret_key")
    VP_PORTAL = config.get("default", "portal")
    VP_WORKSPACE = config.get("default", "workspace", fallback=None)
    return {
        "token": VP_TOKEN,
        "secret_key": VP_SECRET_KEY,
        "portal": VP_PORTAL,
        "workspace": VP_WORKSPACE,
    }


def ssh_connect(ssh_client, username, host_name, port):
    success = False
    password = None
    password_tries = 3
    passphrase = None

    def _connect(host_name, port, username):
        nonlocal password
        nonlocal passphrase
        ssh_client.connect(
            host_name,
            port=port,
            username=username,
            password=password,
            passphrase=passphrase,
        )

    while not success and password_tries > 0:
        try:
            _connect(host_name, port, username)
            break
        except ssh_exception.PasswordRequiredException as e:
            print(e)
            passphrase = getpass.getpass()
        except ssh_exception.AuthenticationException as e:
            password = getpass.getpass(
                f"Password for {username}@{host_name} ({password_tries} tries left): "
            )
            password_tries -= 1
    else:
        raise ssh_exception.AuthenticationException


def _exec_command_remote(ssh_client, command, host, env=None):
    transport = ssh_client.get_transport()
    chan = transport.open_session()
    chan.get_pty()
    stdin = chan.makefile("wb", -1)
    chan.setblocking(0)
    if env:
        chan.update_environment(env)
    chan.exec_command(command)
    while True:
        while chan.recv_ready():
            o = chan.recv(1000).decode("utf-8")
            print(o)
            if "[sudo] password for" in o:
                password = getpass.getpass()
                stdin.write(f"{password}\n")
                stdin.flush()
        while chan.recv_stderr_ready():
            print(chan.recv_stderr(1000).decode("utf-8"))
        if chan.exit_status_ready():
            break
        time.sleep(1)

    retcode = chan.recv_exit_status()
    if retcode != 0:
        raise Exception(f'Failed to installl agent on host "{host}"')


def _win_get_computer_list_ad(base_dn, ad_domain, username, password, use_ssl=False):
    ldap = "LDAP" if not use_ssl else "LDAPS"
    if ad_domain and base_dn:
        ldap_url = f"{ldap}://{ad_domain}/{base_dn}"
    elif ad_domain:
        ldap_url = f"{ldap}://{ad_domain}"
    elif base_dn:
        ldap_url = f"{ldap}://{base_dn}"
    else:
        ldap_url = ""

    log.debug("LDAP url is %s" % (ldap_url))

    ad = (
        ActiveDirectory(ldap_url)
        if not username or not password
        else ActiveDirectory(ldap_url, dn=username, secret=password)
    )
    return [
        str(computer.dNSHostName)
        for computer in ad.search_ext_s(filterstr="(&(objectClass=computer))")
    ]


def _win_execute_on_computer(
    computer_name,
    username,
    password,
    install_script_url,
    vp_install_key,
    port,
    use_ssl=False,
):
    log.info(f'Installing agent on "{computer_name}"')
    with WinClient(
        computer_name, username=username, password=password, port=port, ssl=use_ssl
    ) as client:

        output, streams, had_errors = client.execute_ps(
            """
        Set-ExecutionPolicy Bypass -Scope Process -Force; ((New-Object System.Net.WebClient).DownloadString('%s')) | Out-File $env:temp\\vantagepoint_installer.ps1; cd $env:temp; .\\vantagepoint_installer.ps1 -VPInstallKey %s
        cd 'C:\Program Files\\VantagePoint\\bin'; .\\nssm.exe start vantagepoint
                """
            % (install_script_url, vp_install_key)
        )

    if had_errors:
        for error in streams.error:
            print(error.message)
    if output:
        print(output)


def update_vp_config(**kwargs):
    _config = get_vp_config()
    for k, v in kwargs.items():
        _config[k] = v
    config = configparser.ConfigParser()
    config.readfp(StringIO("[default]"))

    for k, v in _config.items():
        config.set("default", k, v)
    with open(CREDENTIALS_FILE, "w+") as configfile:
        config.write(configfile)

    os.chmod(CREDENTIALS_FILE, stat.S_IRUSR | stat.S_IWUSR)


class Formatter:
    @staticmethod
    def table(headers, rows):
        term_width, _ = shutil.get_terminal_size(fallback=(100, 0))
        table = Texttable(max_width=term_width)
        table.add_rows([headers] + rows)
        table.set_cols_dtype(["t" for _ in headers])
        table.set_deco(Texttable.HEADER)
        table.set_chars(["-", "|", "+", "-"])
        return table.draw() + "\n"

    @staticmethod
    def image_evaluation(evaluation):
        latest_evaluation = next(iter(evaluation.values()))[0]
        problems_found = latest_evaluation["detail"]["result"]["evaluation_problems"]
        result = next(iter(latest_evaluation["detail"]["result"]["result"].values()))[
            "result"
        ]
        if result["final_action"] == "warn":
            result["final_action"] = colored(result["final_action"].upper(), "yellow")

        _formatted = f"""
Image Digest:       {latest_evaluation['detail']['result']['image_digest']}
Status:             {latest_evaluation['status']}
Last Analysis:      {latest_evaluation['last_evaluation']}
Policy:             {latest_evaluation['detail']['policy']['comment']}
Tag:                {latest_evaluation['detail']['result']['tag']}

Scan results:

{Formatter.table(result['header'], result['rows'])}

---------------------------
Final action:       {result['final_action']}
{len(problems_found)} Problems found
        """

        return _formatted

    @staticmethod
    def integration(data):
        return json.dumps(data, indent=4)

    @staticmethod
    def reputation(data):
        risk_levels = ["Informational", "Low", "Medium", "High", "Critical"]
        reputation = data
        reputation["id"] = reputation["id"]
        reputation["risk_score"] = risk_levels[reputation["risk_score"]]
        if reputation["type"] == "DYNAMIC":
            reputation.update({"url": reputation["source"]})
            del reputation["entries"]
        del reputation["source"]
        return json.dumps(reputation, indent=4)


def loadjson_schema(name, version="v1"):
    schema_dir = os.path.join(
        os.path.dirname(__file__), "schemas", f"{name}_schema_{version}.json"
    )
    with open(schema_dir, "r") as f:
        schema = json.load(f)
    return schema


@contextmanager
def edit_entry(entry, suffix=None):
    f = tempfile.NamedTemporaryFile(suffix=suffix)
    f.write(str.encode(entry))
    f.seek(0)
    subprocess.Popen([VP_CLI_DEFAULT_EDITOR, f.name]).wait()
    yield f
    f.close()


def pretty_date(time):
    """
    Get a datetime object or a int() Epoch timestamp and return a
    pretty string like 'an hour ago', 'Yesterday', '3 months ago',
    'just now', etc
    """
    now = datetime.utcnow()
    if isinstance(time, int):
        time = datetime.fromtimestamp(time)
    else:
        time = dateparser.parse(time)

    diff = now - time.replace(tzinfo=None)  # Assuming that time is UTC
    is_past = True if diff.total_seconds() > 0 else False

    second_diff = abs(diff.seconds)
    day_diff = abs(diff.days)
    if day_diff < 0 and is_past:
        return "just now"

    if day_diff == 0:
        if second_diff < 10:
            return "%s now" % ("just" if is_past else "from")
        if second_diff < 60:
            return str(second_diff) + " seconds %s" % ("ago" if is_past else "from now")
        if second_diff < 120:
            return "a minute %s" % ("ago" if is_past else "from now")
        if second_diff < 3600:
            return str(second_diff // 60) + " minutes ago"
        if second_diff < 7200:
            return "an hour %s" % ("ago" if is_past else "from now")
        if second_diff < 86400:
            return str(second_diff // 3600) + " hours %s" % (
                "ago" if is_past else "from now"
            )
    if day_diff == 1:
        return "%s" % ("Yesterday" if is_past else "Tomorrow")
    if day_diff < 7:
        return str(day_diff) + " days %s" % ("ago" if is_past else "from now")
    if day_diff < 31:
        return str(day_diff // 7) + " %s %s" % (
            "weeks" if day_diff > 1 else "week",
            "ago" if is_past else "from now",
        )
    if day_diff < 365:
        return str(day_diff // 30) + " %s %s" % (
            "months" if (day_diff // 30) > 1 else "month",
            "ago" if is_past else "from now",
        )
    return str(day_diff // 365) + " %s %s" % (
        "years" if (day_diff // 365) > 1 else "year",
        "ago" if is_past else "from now",
    )


spinner = itertools.cycle(["-", "/", "|", "\\"])



import os
import json
import tempfile
import subprocess
import json
from jsonschema.validators import Draft4Validator
from cli.apiexternal import (
    get_reputation_feeds,
    add_reputation_feed,
    update_reputaiton_feed,
    delete_reputation_feed,
)
from cli.util import Formatter, loadjson_schema, pretty_date, edit_entry

class Reputation:
    def validate(self, json_file):
        schema = loadjson_schema("reputation")
        validator = Draft4Validator(schema=schema)

        with open(json_file, "r") as f:
            feeds = json.load(f)

        errors = [e.message for e in validator.iter_errors(instance=feeds)]
        if errors:
            raise Exception(
                "\n".join([f'Failed to validate "{json_file}" \n'] + errors)
            )

        return feeds

    @classmethod
    def add(cls, json_file):
        feeds = cls().validate(json_file)

        failed = []
        for feed in feeds["reputation_feeds"]:
            feed["entries"] = ",".join(feed.get("entries", []))
            added = add_reputation_feed(feed)
            if not added:
                failed.append(feed["name"])

        return f"Added {len(feeds) - len(failed)} entries! {len(failed)} failed"

    @staticmethod
    def list_():
        feeds = get_reputation_feeds()
        headers = [
            "Id",
            "Name",
            "Source",
            "Type",
            "Number of indicators",
            "Risk score",
            "Last updated",
            "Status",
        ]

        risk_levels = ["Informational", "Low", "Medium", "High", "Critical"]
        rows = [
            [
                feed["id"],
                feed["name"],
                feed["source"],
                feed["type"],
                feed["number_of_indicators"],
                risk_levels[feed["risk_score"]],
                pretty_date(feed["last_updated"]),
                "Ok"
                if feed["number_of_indicators"] > 0
                else (
                    "No indicators" if feed["number_of_indicators"] == 0 else "Error"
                ),
            ]
            for feed in feeds
        ]
        return Formatter.table(headers, rows)

    @staticmethod
    def get(id_):
        feed = get_reputation_feeds(id_=id_)[0]
        return Formatter.reputation(feed)

    @staticmethod
    def edit(id_):
        feed_formated = json.loads(Formatter.reputation(get_reputation_feeds(id_=id_)[0]))
        for key in ("last_updated", "number_of_indicators", "error"):
            del feed_formated[key]

        with edit_entry(json.dumps(feed_formated, indent=4), suffix=".json") as f:
            edited = json.load(f)

        if edited == feed_formated:
            return "No changes detected!"

        updated = update_reputaiton_feed(id_, edited)
        if updated:
            return "Updated!"
        else:
            raise Exception("Failed!")

    @staticmethod
    def delete(id_):
        return delete_reputation_feed(id_)



import setuptools
from cli.consts import VP_CLI_VERSION

with open("README.md") as f:
    long_description = f.read()

setuptools.setup(
    name="vp-cli",
    version=VP_CLI_VERSION,
    author="VantagePoint",
    author_email="developers@vantagepoint.co",
    description="VantagePoint CLI",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(
        exclude=["tests.*", "tests"],
    ),
   install_requires=[
        "alabaster==0.7.12",
        "attrs==19.3.0",
        "Babel==2.8.0",
        "bcrypt==3.2.0",
        "boto3==1.17",
        "botocore==1.20.0",
        "certifi==2019.11.28",
        "cffi==1.14.4",
        "chardet==5.1.0",
        "checkov==2.3.22",
        "cryptography==3.2.1",
        "dateparser==0.7.6",
        "docopt==0.6.2",
        "docutils==0.16",
        "ezenv==0.91",
        "filelock==3.9.0",
        "idna==2.8",
        "imagesize==1.2.0",
        "importlib-metadata==4.4",
        "Jinja2==2.11.1",
        "jmespath==0.10.0",
        "jsonpointer==2.0",
        "jsonschema==4.6.0",
        "MarkupSafe==1.1.1",
        "mock==3.0.5",
        "more-itertools==8.1.0",
        "orjson==3.8.6",
        "packaging==23.0",
        "paramiko==2.7.2",
        "pluggy==1",
        "py==1.8.1",
        "pycparser==2.20",
        "Pygments==2.5.2",
        "PyNaCl==1.4.0",
        "pyparsing==2.4.6",
        "pypsrp==0.5.0",
        "pyrsistent==0.16.0",
        "pyspnego==0.1.5",
        "python-dateutil==2.8.1",
        "pytz==2019.3",
        "PyYAML==5.4.1",
        "regex==2022.1.18",
        "requests==2.28.2",
        "s3transfer==0.3.3",
        "six==1.14.0",
        "snowballstemmer==2.0.0",
        "termcolor==1.1.0",
        "texttable==1.6.2",
        "toml==0.10.0",
        "tox==4.4.5",
        "tzlocal==2.1",
        "urllib3==1.26.0",
        "wcwidth==0.1.8",
        "zipp==1.0.0",
        "activedirectory==0.14",
# install me pycurl

        ],
    include_package_data=True,
    entry_points={
        "console_scripts": ["vp-cli=cli.main:main"],
    },
    classifiers=[
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
    ],
    python_requires=">=3.7",
)



from abc import ABCMeta, abstractstaticmethod
from cli.apiexternal import file_and_event_search
from cli.util import Formatter


class FileEventSearchMeta(ABCMeta):
    def __new__(cls, name, bases, attr):
        if (
            "FileEventSearch" in [c.__qualname__ for c in bases]
            and "ENDPOINT" not in attr
        ):
            raise AttributeError(f'ENDPOINT not defined for class "{name}"')
        return super().__new__(cls, name, bases, attr)


class FileEventSearch(metaclass=FileEventSearchMeta):
    def search(self, query):
        return self._format_output(file_and_event_search(self.ENDPOINT, query))

    @abstractstaticmethod
    def _format_output(data):
        pass


class CloudLogs(FileEventSearch):
    ENDPOINT = "/api/search/logs/cloud"

    @staticmethod
    def _format_output(data):
        headers = [
            "Id",
            "Aws Region",
            "Event Type",
            "Event Name",
            "Event Time",
            "User Agent",
            "Source IPAddress",
            "Event Source",
            "Event Version",
        ]
        rows = [
            [
                row["meta"]["id"],
                row["awsRegion"],
                row["eventType"],
                row["eventName"],
                row["eventTime"],
                row["userAgent"],
                row["sourceIPAddress"],
                row["eventSource"],
                row["eventVersion"],
            ]
            for row in data["results"]
        ]
        return Formatter.table(headers, rows)


class SysLog(FileEventSearch):
    ENDPOINT = "/api/search/syslog"

    @staticmethod
    def _format_output(data):
        headers = [
            "Id",
            "Host Uuid",
            "Host Name",
            "IPV4 Address",
            "Host OS",
            "Text",
        ]

        rows = [
            [
                row["meta"]["id"],
                row["host_uuid"],
                row["hostname"],
                row["ipv4_addr"],
                row["host_os"],
                (row["text"][:75] + "...") if len(row["text"]) > 75 else row["text"],
            ]
            for row in data["results"]
        ]
        return Formatter.table(headers, rows)


class LogFiles(FileEventSearch):
    ENDPOINT = "/api/search/logs"

    @staticmethod
    def _format_output(data):
        headers = [
            "Id",
            "File",
            "Host UUID",
            "Directory",
            "Date Created",
            "Created By",
            "Last Modified",
        ]

        rows = [
            [
                row["meta"]["id"],
                f"{row['directory']}/{row['file_name']}",
                row["host_uuid"],
                row["directory"],
                row["created_on"],
                row["created_by"],
                row["last_modified_on"],
            ]
            for row in data["results"]
        ]
        return Formatter.table(headers, rows)


class VersionedFiles(FileEventSearch):
    ENDPOINT = "/api/search/versioned"

    @staticmethod
    def _format_output(data):
        headers = [
            "Id",
            "File",
            "Host UUID",
            "Directory",
            "Date Created",
            "Created By",
            "Last Modified",
        ]

        rows = [
            [
                row["meta"]["id"],
                f"{row['directory']}/{row['file_name']}",
                row["host_uuid"],
                row["directory"],
                row["created_on"],
                row["created_by"],
                row["last_modified_on"],
            ]
            for row in data["results"]
        ]
        return Formatter.table(headers, rows)


class WindowsEvents(FileEventSearch):
    ENDPOINT = "/api/search/windows"

    @staticmethod
    def _format_output(data):
        headers = [
            "Id",
            "Generated From",
            "Log Type",
            "Username",
            "User Domain",
            "Message",
            "Host UUID",
            "Host Name",
            "Host OS",
            "OS Version",
            "IPV4 Address",
            "Date Created",
        ]

        rows = [
            [
                row["meta"]["id"],
                row["generated_from"],
                row["log_type"],
                row["username"],
                row["user_domain"],
                (row["message"][:75] + "...").replace("\t", "")
                if len(row["message"]) > 75
                else row["message"].replace("\t", ""),
                row["host_uuid"],
                row["hostname"],
                row["os"],
                row["os_version"],
                row["ipv4_addr"],
                row["created_on"],
            ]
            for row in data["results"]
        ]
        return Formatter.table(headers, rows)
